#include "Radio.h"
#include <avr/wdt.h>
#include <MsTimer2.h>

#define TX_FREQ 10000

#define UNKNOWN 0x0
#define HEURES_PLEINES 0x1
#define HEURES_CREUSES 0x2

#define NODE_ID 20

// B,U,U,U,U,h,h,h,B,B,H,H,H,H,H,H
typedef struct
{
    uint8_t ltarf;
    uint32_t east;
    uint32_t easf01;
    uint32_t easf02;
    uint32_t eait;
    uint16_t urms1;
    uint16_t urms2;
    uint16_t urms3;
    uint8_t pref;
    uint8_t pcoup;
    uint16_t sinsts;
    uint16_t sinsts1;
    uint16_t sinsts2;
    uint16_t sinsts3;
    uint16_t sinsti;
    uint16_t smaxin;

} TIC;
TIC tic = {
        0, 100,200,300,0,231,232,234,12,13,4200,1200,1400,1600,0,0
};

// Must be between 0 & 7
#define PIN_LED_1 3
#define PIN_LED_2 4

Radio radio;

unsigned long lastTx = 0ul;
volatile uint8_t ledCount;

void emonSend(TIC &tic);
void ledOff();
void blink(uint8_t count);

void setup()
{
    Serial.begin(9600);
    wdt_enable(WDTO_4S);
    Serial.println("Initializing radio...");
    radio.init(NODE_ID);
    Serial.println("done.");

    pinMode(PIN_LED_1, OUTPUT);
    pinMode(PIN_LED_2, OUTPUT);
    MsTimer2::set(250, ledOff);

    for (int i = 0; i < 4; i++) {
        digitalWrite(PIN_LED_1, HIGH);
        delay(250);
        digitalWrite(PIN_LED_1, LOW);
        delay(250);
    }

    blink(2);
}

void blink(uint8_t count)
{
    ledCount = count + 1;
    digitalWrite(PIN_LED_2, HIGH);
    MsTimer2::start();
}


void ledOff()
{
    PORTD ^= _BV(PIN_LED_2);
    if (--ledCount == 0) {
          MsTimer2::stop();
    }
}

void loop()
{
    wdt_reset();
    if (millis() - lastTx > TX_FREQ) {
        Serial.println("Sending...");
        emonSend(&tic);
        tic.east += 1;
        tic.easf01 += 1;
        tic.easf02 += 1;
        blink(2);
        Serial.println("sent.");
        lastTx = millis();
    }
}

void emonSend(TIC *tic)
{
    radio.send(tic, sizeof(TIC));
    wdt_reset();
    delay(200);
}
